@@ -26,7 +26,12 @@
 
 G_BEGIN_DECLS
 
-#ifdef PANGO_ENABLE_BACKEND
+/* All of this is deprecated and entirely useless for bindings.
+ * Leave it out of the gir file.
+ */
+#ifndef __GI_SCANNER__
+
+#ifndef PANGO_DISABLE_DEPRECATED
 
 typedef struct _PangoMap PangoMap;
 typedef struct _PangoMapEntry PangoMapEntry;
@@ -40,7 +45,7 @@ typedef struct _PangoIncludedModule PangoIncludedModule;
  * @exit: a function to finalize the module.
  * @create: a function to create an engine, given the engine name.
  *
- * The #PangoIncludedModule structure for a statically linked module
+ * The `PangoIncludedModule` structure for a statically linked module
  * contains the functions that would otherwise be loaded from a dynamically
  * loaded module.
  *
@@ -55,22 +60,24 @@ struct _PangoIncludedModule
   PangoEngine *(*create) (const char       *id);
 };
 
-G_DEPRECATED
+PANGO_DEPRECATED_IN_1_38
 PangoMap *     pango_find_map        (PangoLanguage       *language,
 				      guint                engine_type_id,
 				      guint                render_type_id);
-G_DEPRECATED
+PANGO_DEPRECATED_IN_1_38
 PangoEngine *  pango_map_get_engine  (PangoMap            *map,
 				      PangoScript          script);
-G_DEPRECATED
+PANGO_DEPRECATED_IN_1_38
 void           pango_map_get_engines (PangoMap            *map,
 				      PangoScript          script,
 				      GSList             **exact_engines,
 				      GSList             **fallback_engines);
-G_DEPRECATED
+PANGO_DEPRECATED_IN_1_38
 void           pango_module_register (PangoIncludedModule *module);
 
-#endif /* PANGO_ENABLE_BACKEND */
+#endif /* PANGO_DISABLE_DEPRECATED */
+
+#endif /* __GI_SCANNER__ */
 
 G_END_DECLS
 
