@@ -66,9 +66,10 @@ CODEBLOCK_END_RE = re.compile(
 
 LINK_RE = re.compile(
     r'''
+    (?P<text>\[ [\w\s,\-_:]+ \])?
     \[
     (`)?
-    (?P<fragment>alias|class|const|ctor|enum|error|flags|func|id|iface|method|property|signal|struct|vfunc)
+    (?P<fragment>[\w]+)
     @
     (?P<endpoint>[\w\-_:\.]+)
     (`)?
@@ -119,6 +120,7 @@ LANGUAGE_MAP = {
     'css': 'css',
     'plain': 'plain',
     'xml': 'xml',
+    'javascript': 'javascript',
 }
 
 MD_EXTENSIONS = [
@@ -195,6 +197,7 @@ class LinkGenerator:
         self._fragment = kwargs.get('fragment', '')
         self._endpoint = kwargs.get('endpoint', '')
         self._no_link = kwargs.get('no_link', False)
+        self._alt_text = kwargs.get('text')
 
         assert self._namespace is not None
 
@@ -204,6 +207,7 @@ class LinkGenerator:
 
         fragment_parsers = {
             "alias": self._parse_type,
+            "callback": self._parse_type,
             "class": self._parse_type,
             "const": self._parse_type,
             "ctor": self._parse_method,
@@ -217,12 +221,13 @@ class LinkGenerator:
             "property": self._parse_property,
             "signal": self._parse_signal,
             "struct": self._parse_type,
+            "type": self._parse_type,
             "vfunc": self._parse_method,
         }
 
         parser_method = fragment_parsers.get(self._fragment)
         if parser_method is not None:
-            res = parser_method()
+            res = parser_method(self._fragment)
             if res is not None:
                 self._fragment = None
                 log.warning(str(res))
@@ -232,29 +237,38 @@ class LinkGenerator:
                                            self._fragment, self._endpoint,
                                            "Unable to parse link")))
 
-    def _parse_id(self):
-        t = self._namespace.find_symbol(self._endpoint)
+    def _parse_id(self, fragment):
+        symbol = self._repository.find_symbol(self._endpoint)
+        if symbol is None:
+            return LinkParseError(self._line, self._start, self._end,
+                                  self._fragment, self._endpoint,
+                                  f"Unable to find symbol {self._endpoint}")
+        (ns, t) = symbol
         if isinstance(t, gir.Class) or \
            isinstance(t, gir.Interface) or \
            isinstance(t, gir.Record):
+            self._external = ns is not self._namespace
+            self._ns = ns.name
             self._fragment = 'method'
             self._symbol_name = f"{self._endpoint}()"
             self._name = t.name
-            self._method_name = self._endpoint.replace(self._namespace.symbol_prefix[0] + '_', '')
+            self._method_name = self._endpoint.replace(ns.symbol_prefix[0] + '_', '')
             self._method_name = self._method_name.replace(t.symbol_prefix + '_', '')
             return None
         elif isinstance(t, gir.Function):
+            self._external = ns is not self._namespace
+            self._ns = ns.name
             self._fragment = 'func'
             self._symbol_name = f"{self._endpoint}()"
             self._name = None
-            self._func_name = self._endpoint.replace(self._namespace.symbol_prefix[0] + '_', '')
+            self._func_name = self._endpoint.replace(ns.symbol_prefix[0] + '_', '')
             return None
         else:
             return LinkParseError(self._line, self._start, self._end,
                                   self._fragment, self._endpoint,
-                                  f"Unable to find symbol {self._endpoint}")
+                                  f"Unsupported symbol {self._endpoint}")
 
-    def _parse_type(self):
+    def _parse_type(self, fragment):
         res = TYPE_RE.match(self._endpoint)
         if res:
             ns = res.group('ns')
@@ -286,6 +300,29 @@ class LinkGenerator:
                 return None
         t = namespace.find_real_type(name)
         if t is not None and t.base_ctype is not None:
+            if fragment == 'type':
+                if isinstance(t, gir.Alias):
+                    self._fragment = 'alias'
+                elif isinstance(t, gir.BitField):
+                    self._fragment = 'flags'
+                elif isinstance(t, gir.Callback):
+                    self._fragment = 'callback'
+                elif isinstance(t, gir.Class):
+                    self._fragment = 'class'
+                elif isinstance(t, gir.Constant):
+                    self._fragment = 'const'
+                elif isinstance(t, gir.Enumeration):
+                    self._fragment = 'enum'
+                elif isinstance(t, gir.ErrorDomain):
+                    self._fragment = 'error'
+                elif isinstance(t, gir.Interface):
+                    self._fragment = 'iface'
+                elif isinstance(t, gir.Record) or isinstance(t, gir.Union):
+                    self._fragment = 'struct'
+                else:
+                    return LinkParseError(self._line, self._start, self._end,
+                                          self._fragment, self._endpoint,
+                                          f"Invalid type {t} for '{ns}.{name}'")
             self._name = name
             self._type = t.base_ctype
             return None
@@ -294,7 +331,7 @@ class LinkGenerator:
                                   self._fragment, self._endpoint,
                                   f"Unable to find type '{ns}.{name}'")
 
-    def _parse_property(self):
+    def _parse_property(self, fragment):
         res = PROPERTY_RE.match(self._endpoint)
         if res:
             ns = res.group('ns')
@@ -342,7 +379,7 @@ class LinkGenerator:
                                   self._fragment, self._endpoint,
                                   f"Invalid property '{pname}' for type '{ns}.{name}'")
 
-    def _parse_signal(self):
+    def _parse_signal(self, fragment):
         res = SIGNAL_RE.match(self._endpoint)
         if res:
             ns = res.group('ns')
@@ -390,7 +427,7 @@ class LinkGenerator:
                                   self._fragment, self._endpoint,
                                   f"Invalid signal name '{sname}' for type '{ns}.{name}'")
 
-    def _parse_method(self):
+    def _parse_method(self, fragment):
         res = METHOD_RE.match(self._endpoint)
         if res:
             ns = res.group('ns')
@@ -429,7 +466,7 @@ class LinkGenerator:
             if isinstance(t, gir.Record) and t.struct_for is not None:
                 self._name = t.struct_for
                 self._fragment = "class_method"
-            elif self._fragment == "vfunc" and t.type_struct is not None:
+            elif fragment == "vfunc" and t.type_struct is not None:
                 self._name = name
                 self._type = t.type_struct
             else:
@@ -438,17 +475,17 @@ class LinkGenerator:
             return LinkParseError(self._line, self._start, self._end,
                                   self._fragment, self._endpoint,
                                   f"Unable to find type '{ns}.{name}'")
-        if self._fragment == "ctor":
+        if fragment == "ctor":
             methods = getattr(t, "constructors", [])
-        elif self._fragment in ["method", "class_method"]:
+        elif fragment in ["method", "class_method"]:
             methods = getattr(t, "methods", [])
-        elif self._fragment == "vfunc":
+        elif fragment == "vfunc":
             methods = getattr(t, "virtual_methods", [])
         else:
             methods = []
         for m in methods:
             if m.name == method:
-                if self._fragment == "vfunc":
+                if fragment == "vfunc":
                     self._vfunc_name = m.name
                 else:
                     self._symbol_name = f"{m.identifier}()"
@@ -457,7 +494,7 @@ class LinkGenerator:
                               self._fragment, self._endpoint,
                               f"Unable to find method '{ns}.{name}.{method}'")
 
-    def _parse_func(self):
+    def _parse_func(self, fragment):
         tokens = self._endpoint.split('.')
         # Case 1: [func@init] => gtk_init()
         if len(tokens) == 1:
@@ -530,7 +567,9 @@ class LinkGenerator:
 
     @property
     def text(self):
-        if self._fragment in ['alias', 'class', 'const', 'enum', 'error', 'flags', 'iface', 'struct']:
+        if self._alt_text is not None:
+            return self._alt_text[1:len(self._alt_text) - 1]
+        elif self._fragment in ['alias', 'class', 'const', 'enum', 'error', 'flags', 'iface', 'struct']:
             return f"<code>{self._type}</code>"
         elif self._fragment == 'property':
             return f"<code>{self._type}:{self._property_name}</code>"
@@ -579,11 +618,23 @@ class LinkGenerator:
 
 
 def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plain=False, max_length=10):
+    if plain:
+        text = text.replace('\n', ' ')
+        text = re.sub(r'<[^<]+?>', '', text)
+        if max_length > 0:
+            words = text.split(' ')
+            if len(words) > max_length:
+                words = words[:max_length - 1]
+                words.append('...')
+                text = ' '.join(words)
+        return text
+
     processed_text = []
 
     code_block_text = []
     code_block_language = None
     inside_code_block = False
+
     for line in text.split("\n"):
         # If we're in "summary" mode, we bail out at the first empty line
         # after a paragraph
@@ -623,12 +674,13 @@ def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plai
             for m in LINK_RE.finditer(line, idx):
                 fragment = m.group('fragment')
                 endpoint = m.group('endpoint')
+                text = m.group('text')
                 start = m.start()
                 end = m.end()
                 link = LinkGenerator(line=line, start=start, end=end,
                                      namespace=namespace,
                                      fragment=fragment, endpoint=endpoint,
-                                     no_link=summary)
+                                     no_link=summary, text=text)
                 left_pad = line[idx:start]
                 replacement = re.sub(LINK_RE, str(link), line[start:end])
                 new_line.append(left_pad)
@@ -641,16 +693,20 @@ def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plai
             else:
                 processed_text.append("".join(new_line))
 
-    if plain:
-        text = text.replace('\n', ' ')
-        text = re.sub(r'<[^<]+?>', '', text)
-        if max_length > 0:
-            words = text.split(' ')
-            if len(words) > max_length:
-                words = words[:max_length - 1]
-                words.append('...')
-                text = ' '.join(words)
-        return text
+    if len(processed_text) == 0:
+        return ''
+
+    # Capitalize the first character of the first line, but only if it does not
+    # start with a link or a gtk-doc marker, to avoid messing up the rest of
+    # the string
+    first_line = processed_text[0]
+    if first_line and first_line[0].isalpha():
+        processed_text[0] = ''.join([first_line[0:1].upper(), first_line[1:]])
+
+    # Append a period, if one isn't there already
+    last_line = processed_text[-1]
+    if last_line and last_line[-1].isalpha():
+        processed_text[-1] = ''.join([last_line, '.'])
 
     if md is None:
         md_ext = extensions.copy()
@@ -818,9 +874,21 @@ def default_search_paths():
 
     paths = []
     paths.append(os.getcwd())
+    # Add sys.base_prefix when using MSYS2
+    if sys.platform == 'win32' and 'GCC' in sys.version:
+        paths.append(os.path.join(sys.base_prefix, 'share', 'gir-1.0'))
     if xdg_data_home is not None:
         paths.append(os.path.join(xdg_data_home, "gir-1.0"))
     if xdg_data_dirs is not None:
         paths.extend([os.path.join(x, "gir-1.0") for x in xdg_data_dirs])
 
     return paths
+
+
+def find_extra_content_file(content_dirs, file):
+    for p in content_dirs:
+        full_path = os.path.join(p, file)
+        if os.path.isfile(full_path):
+            return full_path
+
+    raise FileNotFoundError(f"Content file {file} not found in any content directory")
